// TMap.cpp - show details of database table structure

#define INCL_WINSYS           //include WINSYS stuff from os2.h
#include <os2.h>              //WinQuerySysValue
#include <sqlenv.h>           //access to sqlca.sqlcode
#include <iobjwin.hpp>        //IObjectWindow for IMessageBox
#include <imsgbox.hpp>        //IMessageBox for error message
#include <iscroll.hpp>        //systemScrollBarWidth
#include <ipoint.hpp>         //ISize
#include <iapp.hpp>           //IApplication
#include <icnrctl.hpp>        //IContainer control
#include <icnrhdr.hpp>        //ICnrHandler - container control events
#include <icnrcol.hpp>        //IContainerColumn
#include <icolor.hpp>         //IColor
#include <istring.hpp>        //IString
#include <iframe.hpp>         //IFrameWindow
#include <icmdhdr.hpp>        //ICommandEvent& ICommandHandler
#include <ifont.hpp>          //IFont
#include "dbs.h"              //table row structs shared with dbs.sqc
#include "tmap.hpp"           //our class interface definitions

void main(int argc, char *argv[]) {
   if (argc != 2 || *argv[1] == '?') EMsg("format: TMAP dbname");
   Database DB2(argv[1]);           //if error, exit; else, connect
   IString title("Database: "); title += argv[1];
   Tables DB2Tables(&title);        //application's main control panel
   IApplication::current().run();   //start PM message processing loop
}

Tables::Tables(const IString *title):IFrameWindow(*title,SYS_TABLES) {
   table = new IContainerControl(SYS_TAB_ROWS, this, this);
   setClient(table); event.handleEventsFor(table);
   Database::Relations *dbTbl = new Database::Relations();  //open csr
   int rc = dbTbl->next();                                  //get row
   while (rc != NOTFOUND) {
      Creator *creator = new Creator(dbTbl->tRow.creator, this);
      table->addObject(creator);          //first level tree node
      IString buf(dbTbl->tRow.creator);
      while (buf == dbTbl->tRow.creator && rc != NOTFOUND) {
         Name *name = new Name(&dbTbl->tRow, this);
         table->addObject(name, creator); //name; leaf node of creator
         rc = dbTbl->next();              //get next row
      }
   }
   delete dbTbl;                          //close the cursor
   table->setDeleteObjectsOnClose().showTreeTextView();
   IFont font("Helv",10); table->setFont(font);
   ISize xy = desktopWindow()->size(); int w = font.avgCharWidth()*40;
   sizeTo(ISize(w,xy.height() * .96));
   moveTo(IPoint(xy.width() * .98 - w, xy.height() * .02));
   table->setFocus(); show();             //take right side of screen
};

Tables::Name::Name(struct tabRow *tRow, Tables *owner)
      :IContainerObject(IString(tRow->name))
   { creator = tRow->creator; name = tRow->name; };

void Tables::Name::handleOpen(IContainerControl *table) {
   IString title(creator); title += "."; title += name;
   Details *details = new Details(title, creator, name);
};

Details::Details(char *title, char *creator, char *name)
      :IFrameWindow (title, SYS_COLUMNS) {
   table=new IContainerControl(SYS_COL_ROWS, this, this);
   setClient(table); event.handleEventsFor(table);

   cNam=new IContainerColumn(IContainerObject::iconTextOffset());
   cTyp=new IContainerColumn(offsetof(Row,type));
   cLen=new IContainerColumn(offsetof(Row,length));
   cNul=new IContainerColumn(offsetof(Row,nulls));
   cRem=new IContainerColumn(offsetof(Row,rem));

   table->addColumn(&cNam->showSeparators().setHeadingText("name"));
   table->addColumn(&cTyp->showSeparators().setHeadingText("type"));
   table->addColumn(&cLen->showSeparators().setHeadingText("length"));
   table->addColumn(&cNul->showSeparators().setHeadingText("nulls"));
   table->addColumn(&cRem->showSeparators().setHeadingText("remark"));

   table->setDeleteObjectsOnClose().showDetailsView().setFocus();

   IFont font("Helv", 10); table->setFont(font);   //size logic OK if
   int charW = font.avgCharWidth();                //8 <= point <= 14
   int lineH = font.maxCharHeight() + table->lineSpacing();
   int dispH = font.maxCharHeight() * 1.75 + .5;   //column headings

   Database::TabRem *rem = new Database::TabRem();
   char tabRem[255]; rem->getTabRem(creator, name, &tabRem);
   if (tabRem[0] != '\0' ) {
      table->setTitle(tabRem).showTitle().showTitleSeparator();
      dispH += dispH;   //add title height; same as col headings above
   }
   Database::Attributes *att = new Database::Attributes(creator,name);
   int rc = att->next();                     //get first row
   while (rc != NOTFOUND) {
      Row *row = new Row(&att->cRow, this);
      table->addObject(row); dispH += lineH; //allow for display space
      rc = att->next();                      //get next row
   }
   delete att;                               //now close the cursor

   int dispW = cNam->show().displayWidth() + 2.5 * charW;   //name
   table->setDetailsViewSplit(cNam, dispW);                 //splitbar
   dispW += cTyp->show().displayWidth() + 3 * charW         //etc...
          + cLen->show().displayWidth() + 3 * charW
          + cNul->show().displayWidth()  + 3 * charW
          + WinQuerySysValue(HWND_DESKTOP, SV_CXVSCROLL);
   dispH += WinQuerySysValue(HWND_DESKTOP, SV_CYHSCROLL);
   IRectangle xy =
            frameRectFor(IRectangle(IPoint(0,0),ISize(dispW,dispH)));
   sizeTo(xy.topRight() - xy.bottomLeft()); show();
};

Details::Row::Row(struct colRow *cRow, Details *owner)
      :IContainerObject(IString(cRow->name)) {
   type = cRow->type; length = cRow->length;
   nulls = cRow->nulls; rem = cRow->rem;
};

void Details::Row::handleOpen(IContainerControl *table)
{
   return;
};

EMsg::EMsg():IObjectWindow() {
   char msg[512];
   if (sqlaintp(msg, 512, 0, &sqlca) < 0)
      strcpy(msg,"SQLAINTP error - can't retrieve message text");
   IMessageBox mbx(this);
   mbx.show(msg,mbx.cancelButton|mbx.errorIcon|mbx.applicationModal);
   exit(12);
};

EMsg::EMsg(char *msg):IObjectWindow() {
   IMessageBox mbx(this);
   mbx.show(msg,mbx.cancelButton|mbx.errorIcon|mbx.applicationModal);
   exit(12);
};

