//TMap.hpp - show details of database table structure

#ifndef TMAP_HPP
#define TMAP_HPP

#define SYS_TABLES      0x2000      //frame window - sysibm.systables
#define SYS_TAB_ROWS    0x2010      //container - sysibm.systables
#define SYS_COLUMNS     0x2100      //frame window - sysibm.syscolumns
#define SYS_COL_ROWS    0x2110      //container - sysibm.syscolumns

extern struct sqlca sqlca;          //for sqlca.sqlcode
extern "c" {
   int connect(char *dbname);       //connect to dbname
   void reset();
   int openTabRow();
   int fetchTabRow(struct tabRow *tRow);
   void closeTabRow();
   int selectTabRem(char *creator, char *name, char (*tabRem)[255]);
   int openColRow(char *creator, char *name);
   int fetchColRow(struct colRow *cRow);
   void closeColRow();
}

class EMsg:public IObjectWindow {   //IMessagebox for error, exit
   public:
      EMsg();                       //show SQL error message text
      EMsg(char *msg);              //show our error string
};

class Database {
   public:
      Database(char *db) { if (connect(db)) EMsg(); }
      ~Database() { reset(); }
      class Relations {    //provider for Tables from sysibm.systables
         public:
            Relations() { if(openTabRow()) EMsg(); }
            int next() { return fetchTabRow(&tRow); }
            ~Relations() { closeTabRow(); }
            struct tabRow tRow;
      };
      class Attributes {   //supplies Details from sysibm.syscolumns
         public:
            Attributes(char *creator, char *name)
                  { if(openColRow(creator, name)) EMsg(); }
            int next() { return fetchColRow(&cRow); }
            ~Attributes() { closeColRow(); }
            struct colRow cRow;
      };
      class TabRem {       //remark for Details from sysibm.systables
         public:
            void getTabRem(char *creator, char *name, char (*tr)[255])
                  { if (selectTabRem(creator, name, tr)) EMsg(); }
      };
};

class Tables:public IFrameWindow {  //tree view of creator, table name
   public:
      Tables(const IString *title);
      void showColumns(struct tabRow& tRow);
      class Creator:public IContainerObject {   //table creator object
         friend class Tables;
         public:
            Creator(char *creator, Tables *owner)
               :IContainerObject(IString(creator)) {};
            void handleOpen(IContainerControl *w) { w->expand(this); }
      };
      class Name:public IContainerObject {      //table name object
         friend class Tables;
         public:
            Name(struct tabRow *tRow, Tables *owner);
            void handleOpen(IContainerControl *w);
         private:
            IString creator, name;
      };
   private:
      IContainerControl *table;
      ICnrHandler event;
};

class Details:public IFrameWindow { //details view of table attributes
   public:
      Details(char *title, char *creator, char *name);
      void showColumns() { show(); }
      class Row:public IContainerObject {       //column details
         friend class Details;
         public:
            Row(struct colRow *cRow, Details *owner);
            void handleOpen(IContainerControl *w);
         private:
            IString type, length, nulls, rem;
      };
   private:
      IContainerControl *table;
      ICnrHandler event;
      IContainerColumn *cNam, *cTyp, *cLen, *cNul, *cRem;
};
#endif

